@@ -1,8 +1,6 @@
-{-# LANGUAGE FlexibleInstances #-}
-
 -- |
 -- Module      : Database.Relational.Internal.String
--- Copyright   : 2014-2017 Kei Hibino
+-- Copyright   : 2014-2019 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -16,8 +14,6 @@ module Database.Relational.Internal.String (
   rowStringSQL, rowPlaceHolderStringSQL,
 
   rowConsStringSQL, listStringSQL,
-
-  boolSQL,
   ) where
 
 import Language.SQL.Keyword (Keyword, word, wordShow, fold, (|*|), paren)
@@ -52,11 +48,3 @@ rowConsStringSQL =  paren . fold (|*|)
 -- | List String of SQL.
 listStringSQL :: [StringSQL] -> StringSQL
 listStringSQL =  paren . fold (|*|)
-
--- | SQL expressions for Bool type.
-boolSQL :: Bool -> StringSQL
-boolSQL =
-    stringSQL . d
-  where
-    d True  = "(0=0)"
-    d False = "(0=1)"
