@@ -1,6 +1,6 @@
 -- |
 -- Module      : Database.Relational.Pi
--- Copyright   : 2013-2017 Kei Hibino
+-- Copyright   : 2013-2019 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -14,11 +14,15 @@ module Database.Relational.Pi (
   Pi, (<.>), (<?.>), (<?.?>),
 
   id',
+
+  -- * Low-level API
+  expandIndexes', expandIndexes,
   ) where
 
 import qualified Control.Category as Category
 
-import Database.Relational.Pi.Unsafe (Pi, (<.>), (<?.>), (<?.?>))
+import Database.Relational.Pi.Unsafe
+  (Pi, (<.>), (<?.>), (<?.?>), expandIndexes', expandIndexes)
 
 
 -- | Identity projection path.
