@@ -3,31 +3,25 @@
 
 -- |
 -- Module      : Database.Relational.Pure
--- Copyright   : 2013-2017 Kei Hibino
+-- Copyright   : 2013-2019 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
 -- Stability   : experimental
 -- Portability : unknown
 --
--- This module defines interfaces between haskell pure values
--- and query internal record values.
+-- This module defines instances which lift from haskell pure values
+-- to query internal record values.
 module Database.Relational.Pure () where
 
 import Control.Applicative (pure)
-import Data.Monoid ((<>))
 import Data.Int (Int8, Int16, Int32, Int64)
 import Data.Word (Word8, Word16, Word32, Word64, Word)
-import Data.ByteString (ByteString)
-import qualified Data.ByteString.Lazy as LB
 import Data.Text (Text)
 import qualified Data.Text as T
-import qualified Data.Text.Encoding as T
 import qualified Data.Text.Lazy as LT
-import qualified Data.Text.Lazy.Encoding as LT
 import Text.Printf (PrintfArg, printf)
-import Data.Time (FormatTime, Day, TimeOfDay, LocalTime, UTCTime, ZonedTime, formatTime)
-import Data.Time.Locale.Compat (defaultTimeLocale)
+import Data.Time (Day, TimeOfDay, LocalTime)
 import Data.DList (DList, fromList)
 
 import Language.SQL.Keyword (Keyword (..))
@@ -36,104 +30,79 @@ import Database.Record
 import Database.Record.Persistable
   (runPersistableRecordWidth)
 
-import Database.Relational.Internal.String (StringSQL, stringSQL, boolSQL)
+import Database.Relational.Internal.String (StringSQL, stringSQL)
+import qualified Database.Relational.Internal.Literal as Lit
 
 import Database.Relational.ProjectableClass (LiteralSQL (..))
 
 
--- | Constant integral SQL terms.
-intTermsSQL :: (Show a, Integral a) => a -> DList StringSQL
-intTermsSQL =  pure . stringSQL . show
-
--- | Escape 'String' for constant SQL string expression.
-escapeStringToSqlExpr :: String -> String
-escapeStringToSqlExpr =  rec  where
-  rec ""        = ""
-  rec ('\'':cs) = '\'' : '\'' : rec cs
-  rec (c:cs)    = c : rec cs
-
--- | From 'String' into constant SQL string expression.
-stringExprSQL :: String -> StringSQL
-stringExprSQL =  stringSQL . ('\'':) . (++ "'") . escapeStringToSqlExpr
-
-stringTermsSQL :: String -> DList StringSQL
-stringTermsSQL = pure . stringExprSQL
-
 -- | Constant SQL terms of '()'.
 instance LiteralSQL ()
 
 -- | Constant SQL terms of 'Int8'.
 instance LiteralSQL Int8 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Int16'.
 instance LiteralSQL Int16 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Int32'.
 instance LiteralSQL Int32 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Int64'.
 instance LiteralSQL Int64 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Int'.
 --   Use this carefully, because this is architecture dependent size of integer type.
 instance LiteralSQL Int where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Word8'.
 instance LiteralSQL Word8 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Word16'.
 instance LiteralSQL Word16 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Word32'.
 instance LiteralSQL Word32 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Word64'.
 instance LiteralSQL Word64 where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'Word'.
 --   Use this carefully, because this is architecture dependent size of integer type.
 instance LiteralSQL Word where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 instance LiteralSQL Integer where
-  showLiteral' = intTermsSQL
+  showLiteral' = pure . Lit.integral
 
 -- | Constant SQL terms of 'String'.
 instance LiteralSQL String where
-  showLiteral' = stringTermsSQL
-
--- | Constant SQL terms of 'ByteString'.
-instance LiteralSQL ByteString where
-  showLiteral' = stringTermsSQL . T.unpack . T.decodeUtf8
-
--- | Constant SQL terms of 'LB.ByteString'.
-instance LiteralSQL LB.ByteString where
-  showLiteral' = stringTermsSQL . LT.unpack . LT.decodeUtf8
+  showLiteral' = pure . Lit.stringExpr
 
 -- | Constant SQL terms of 'Text'.
 instance LiteralSQL Text where
-  showLiteral' = stringTermsSQL . T.unpack
+  showLiteral' = pure . Lit.stringExpr . T.unpack
 
 -- | Constant SQL terms of 'LT.Text'.
 instance LiteralSQL LT.Text where
-  showLiteral' = stringTermsSQL . LT.unpack
+  showLiteral' = pure . Lit.stringExpr . LT.unpack
 
 -- | Constant SQL terms of 'Char'.
 instance LiteralSQL Char where
-  showLiteral' = stringTermsSQL . (:"")
+  showLiteral' = pure . Lit.stringExpr . (:"")
 
 -- | Constant SQL terms of 'Bool'.
 instance LiteralSQL Bool where
-  showLiteral' = pure . boolSQL
+  showLiteral' = pure . Lit.bool
 
 floatTerms :: (PrintfArg a, Ord a, Num a)=> a -> DList StringSQL
 floatTerms f = pure . stringSQL $ printf fmt f  where
@@ -149,30 +118,17 @@ instance LiteralSQL Float where
 instance LiteralSQL Double where
   showLiteral' = floatTerms
 
-constantTimeTerms :: FormatTime t => Keyword -> String -> t -> DList StringSQL
-constantTimeTerms kw fmt t = pure $ kw <> stringExprSQL (formatTime defaultTimeLocale fmt t)
-
 -- | Constant SQL terms of 'Day'.
 instance LiteralSQL Day where
-  showLiteral' = constantTimeTerms DATE "%Y-%m-%d"
+  showLiteral' = pure . Lit.timestamp DATE "%Y-%m-%d"
 
 -- | Constant SQL terms of 'TimeOfDay'.
 instance LiteralSQL TimeOfDay where
-  showLiteral' = constantTimeTerms TIME "%H:%M:%S"
+  showLiteral' = pure . Lit.timestamp TIME "%H:%M:%S"
 
 -- | Constant SQL terms of 'LocalTime'.
 instance LiteralSQL LocalTime where
-  showLiteral' = constantTimeTerms TIMESTAMP "%Y-%m-%d %H:%M:%S"
-
--- | Constant SQL terms of 'ZonedTime'.
---   This generates ***NOT STANDARD*** SQL of TIMESTAMPTZ literal.
-instance LiteralSQL ZonedTime where
-  showLiteral' = constantTimeTerms TIMESTAMPTZ "%Y-%m-%d %H:%M:%S%z"
-
--- | Constant SQL terms of 'UTCTime'.
---   This generates ***NOT STANDARD*** SQL of TIMESTAMPTZ literal with UTC timezone.
-instance LiteralSQL UTCTime where
-  showLiteral' = constantTimeTerms TIMESTAMPTZ "%Y-%m-%d %H:%M:%S%z"
+  showLiteral' = pure . Lit.timestamp TIMESTAMP "%Y-%m-%d %H:%M:%S"
 
 showMaybeTerms :: LiteralSQL a => PersistableRecordWidth a -> Maybe a -> DList StringSQL
 showMaybeTerms wa = d  where
