@@ -2,7 +2,7 @@
 
 -- |
 -- Module      : Database.Relational.SqlSyntax.Fold
--- Copyright   : 2013-2018 Kei Hibino
+-- Copyright   : 2013-2019 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -15,7 +15,7 @@ module Database.Relational.SqlSyntax.Fold (
   showSQL, toSQL, unitSQL, width,
 
   -- * Qualified Sub-query
-  queryWidth,
+  queryWidth, corrSubQueryTerm,
 
   -- * Sub-query columns
   column,
@@ -48,7 +48,8 @@ import Database.Relational.Internal.Config
 import Database.Relational.Internal.UntypedTable ((!))
 import qualified Database.Relational.Internal.UntypedTable as UntypedTable
 import Database.Relational.Internal.String
-  (StringSQL, stringSQL, rowStringSQL, showStringSQL, boolSQL, )
+  (StringSQL, stringSQL, rowStringSQL, showStringSQL, )
+import qualified Database.Relational.Internal.Literal as Lit
 import Database.Relational.SqlSyntax.Types
   (SubQuery (..), Record, Tuple, Predicate,
    Column (..), CaseClause(..), WhenClauses (..),
@@ -94,11 +95,6 @@ c `asColumnN` n =c `SQL.as` columnN n
 columnFromId :: Qualifier -> Int -> StringSQL
 columnFromId qi i = qi <.> columnN i
 
--- | From 'Qualified' SQL string into qualified formed 'String'
---  like (SELECT ...) AS T<n>
-qualifiedSQLas :: Qualified StringSQL -> StringSQL
-qualifiedSQLas q = Syntax.unQualify q <> showQualifier (Syntax.qualifier q)
-
 -- | Width of 'SubQuery'.
 width :: SubQuery -> Int
 width =  d  where
@@ -176,6 +172,17 @@ showSQL = snd . toSQLs
 toSQL :: SubQuery -> String
 toSQL =  showStringSQL . showSQL
 
+-- | Term of qualified table or qualified subquery,
+--   used in join-clause of SELECT, correlated UPDATE and DELETE statements.
+--   When SubQuery is table, expression will be like <TABLE> [AS] T<n>
+corrSubQueryTerm :: Bool                -- ^ if True, add AS keyword. SQLite causes syntax error on UPDATE or DELETE statement.
+                 -> Qualified SubQuery  -- ^ subquery structure with qualifier
+                 -> StringSQL           -- ^ result SQL string
+corrSubQueryTerm addAS qq =
+    showUnitSQL (Syntax.unQualify qq) `asOP` showQualifier (Syntax.qualifier qq)
+  where
+    asOP = if addAS then SQL.as else (<>)
+
 -- | Get column SQL string of 'Qualified' 'SubQuery'.
 column :: Qualified SubQuery -> Int -> StringSQL
 column qs =  d (Syntax.unQualify qs)  where
@@ -244,13 +251,13 @@ showsQueryProduct =  rec  where
   urec n = case Syntax.nodeTree n of
     p@(Leaf _)     -> rec p
     p@(Join {})    -> SQL.paren (rec p)
-  rec (Leaf q)               = qualifiedSQLas $ fmap showUnitSQL q
+  rec (Leaf q)               = uncurry corrSubQueryTerm q
   rec (Join left' right' rs) =
     mconcat
     [urec left',
      joinType (Syntax.nodeAttr left') (Syntax.nodeAttr right'), JOIN,
      urec right',
-     ON, foldr1 SQL.and $ ps ++ concat [ pure $ boolSQL True | null ps ] ]
+     ON, foldr1 SQL.and $ ps ++ concat [ pure $ Lit.bool True | null ps ] ]
     where ps = [ rowStringSQL $ recordRawColumns p | p <- rs ]
 
 -- | Shows join product of query.
