@@ -1,6 +1,6 @@
 -- |
 -- Module      : Database.Relational.Type
--- Copyright   : 2013-2018 Kei Hibino
+-- Copyright   : 2013-2019 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -12,6 +12,7 @@ module Database.Relational.Type (
   -- * Typed query statement
   Query (..), unsafeTypedQuery,
 
+  relationalQuery_,
   relationalQuery', relationalQuery,
 
   relationalQuerySQL,
@@ -28,7 +29,8 @@ module Database.Relational.Type (
   unsafeTypedInsert', unsafeTypedInsert, typedInsert', insert,
   typedInsertValue', insertValue', insertValue, insertValueNoPH,
   insertValueList', insertValueList,
-  InsertQuery (..), unsafeTypedInsertQuery, typedInsertQuery', insertQuery,
+  InsertQuery (..), unsafeTypedInsertQuery,
+  typedInsertQuery', insertQuery', insertQuery,
 
   insertQuerySQL,
 
@@ -41,6 +43,7 @@ module Database.Relational.Type (
   UntypeableNoFetch (..),
 
   -- * Deprecated
+
   typedUpdate,
   typedInsert, typedInsertValue, typedInsertQuery,
   typedDelete,
@@ -56,32 +59,31 @@ module Database.Relational.Type (
   derivedDelete', derivedDelete,
   ) where
 
-import Control.Applicative ((<*))
 import Data.Monoid ((<>))
+import Data.Functor.ProductIsomorphic (peRight)
 
 import Database.Record (PersistableWidth)
 
 import Database.Relational.Internal.Config (Config, defaultConfig)
+import Database.Relational.Internal.ContextType (Flat)
 import Database.Relational.Internal.String (showStringSQL)
+import Database.Relational.SqlSyntax (Record)
 
 import Database.Relational.Monad.BaseType (Relation, sqlFromRelationWith)
-import Database.Relational.Monad.Restrict (RestrictedStatement)
-import Database.Relational.Monad.Assign (AssignStatement)
+import Database.Relational.Monad.Restrict (Restrict)
+import Database.Relational.Monad.Assign (Assign)
 import Database.Relational.Monad.Register (Register)
 import Database.Relational.Relation (tableOf)
 import Database.Relational.Effect
-  (Restriction, restriction, restriction', UpdateTarget, updateTarget',
-   liftTargetAllColumn, liftTargetAllColumn',
-   InsertTarget, insertTarget',
-   sqlWhereFromRestriction, sqlFromUpdateTarget, piRegister,
+  (liftTargetAllColumn', InsertTarget, insertTarget',
+   deleteFromRestriction, updateFromUpdateTarget, piRegister,
    sqlChunkFromInsertTarget, sqlFromInsertTarget, sqlChunksFromRecordList)
 import Database.Relational.Pi (Pi)
 import Database.Relational.Table (Table, TableDerivable, derivedTable)
 import Database.Relational.ProjectableClass (LiteralSQL)
 import Database.Relational.Projectable (PlaceHolders, unitPH)
 import Database.Relational.SimpleSql
-  (QuerySuffix, showsQuerySuffix, insertPrefixSQL,
-   updateOtherThanKeySQL, updatePrefixSQL, deletePrefixSQL)
+  (QuerySuffix, showsQuerySuffix, insertPrefixSQL, updateOtherThanKeySQL, )
 
 
 -- | Query type with place-holder parameter 'p' and query result type 'a'.
@@ -98,11 +100,17 @@ instance Show (Query p a) where
 
 -- | From 'Relation' into untyped SQL query string.
 relationalQuerySQL :: Config -> Relation p r -> QuerySuffix -> String
-relationalQuerySQL config rel qsuf = showStringSQL $ sqlFromRelationWith rel config <> showsQuerySuffix qsuf
+relationalQuerySQL config rel qsuf =
+  showStringSQL $ sqlFromRelationWith rel config <> showsQuerySuffix qsuf
+
+-- | From 'Relation' into typed 'Query' with suffix SQL words.
+relationalQuery_ :: Config -> Relation p r -> QuerySuffix -> Query p r
+relationalQuery_ config rel qsuf =
+  unsafeTypedQuery $ relationalQuerySQL config rel qsuf
 
 -- | From 'Relation' into typed 'Query' with suffix SQL words.
 relationalQuery' :: Relation p r -> QuerySuffix -> Query p r
-relationalQuery' rel qsuf = unsafeTypedQuery $ relationalQuerySQL defaultConfig rel qsuf
+relationalQuery' = relationalQuery_ defaultConfig
 
 -- | From 'Relation' into typed 'Query'.
 relationalQuery :: Relation p r -> Query p r
@@ -152,101 +160,98 @@ newtype Update p = Update { untypeUpdate :: String }
 unsafeTypedUpdate :: String -> Update p
 unsafeTypedUpdate =  Update
 
--- | Make untyped update SQL string from 'Table' and 'UpdateTarget'.
-updateSQL :: Config -> Table r -> UpdateTarget p r -> String
-updateSQL config tbl ut = showStringSQL $ updatePrefixSQL tbl <> sqlFromUpdateTarget config tbl ut
+-- | Make untyped update SQL string from 'Table' and 'Assign' computation.
+updateSQL :: Config -> Table r -> (Record Flat r -> Assign r (PlaceHolders p)) -> String
+updateSQL config tbl ut = showStringSQL $ updateFromUpdateTarget config tbl ut
 
--- | Make typed 'Update' from 'Config', 'Table' and 'UpdateTarget'.
-typedUpdate' :: Config -> Table r -> UpdateTarget p r -> Update p
+-- | Make typed 'Update' from 'Config', 'Table' and 'Assign' computation.
+typedUpdate' :: Config -> Table r -> (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
 typedUpdate' config tbl ut = unsafeTypedUpdate $ updateSQL config tbl ut
 
 {-# DEPRECATED typedUpdate "use `typedUpdate' defaultConfig` instead of this." #-}
--- | Make typed 'Update' using 'defaultConfig', 'Table' and 'UpdateTarget'.
-typedUpdate :: Table r -> UpdateTarget p r -> Update p
+-- | Make typed 'Update' using 'defaultConfig', 'Table' and 'Assign' computation.
+typedUpdate :: Table r -> (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
 typedUpdate =  typedUpdate' defaultConfig
 
-targetTable :: TableDerivable r => UpdateTarget p r -> Table r
+targetTable :: TableDerivable r => (Record Flat r -> Assign r (PlaceHolders p)) -> Table r
 targetTable =  const derivedTable
 
--- | Make typed 'Update' from 'Config', derived table and 'AssignStatement'
-update' :: TableDerivable r => Config -> AssignStatement r (PlaceHolders p) -> Update p
-update' config utc =  typedUpdate' config (targetTable ut) ut  where
-  ut = updateTarget' utc
+-- | Make typed 'Update' from 'Config', derived table and 'Assign' computation.
+update' :: TableDerivable r => Config -> (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
+update' config ac =  typedUpdate' config (targetTable ac) ac
 
 {-# DEPRECATED derivedUpdate' "use `update'` instead of this." #-}
--- | Make typed 'Update' from 'Config', derived table and 'AssignStatement'
-derivedUpdate' :: TableDerivable r => Config -> AssignStatement r (PlaceHolders p) -> Update p
+-- | Make typed 'Update' from 'Config', derived table and 'Assign' computation.
+derivedUpdate' :: TableDerivable r => Config -> (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
 derivedUpdate' = update'
 
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'
-update :: TableDerivable r => AssignStatement r (PlaceHolders p) -> Update p
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'Assign' computation.
+update :: TableDerivable r => (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
 update = update' defaultConfig
 
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement' with no(unit) placeholder.
-updateNoPH :: TableDerivable r => AssignStatement r () -> Update ()
-updateNoPH af = update $ (return unitPH <*) . af
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'Assign' computation with no(unit) placeholder.
+updateNoPH :: TableDerivable r => (Record Flat r -> Assign r ()) -> Update ()
+updateNoPH af = update $ (>> return unitPH) . af
 
 {-# DEPRECATED derivedUpdate "use `update` instead of this." #-}
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'
-derivedUpdate :: TableDerivable r => AssignStatement r (PlaceHolders p) -> Update p
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'Assign' computation.
+derivedUpdate :: TableDerivable r => (Record Flat r -> Assign r (PlaceHolders p)) -> Update p
 derivedUpdate = update
 
 
--- | Make typed 'Update' from 'Config', 'Table' and 'Restriction'.
+-- | Make typed 'Update' from 'Config', 'Table' and 'Restrict' computation.
 --   Update target is all column.
 typedUpdateAllColumn' :: PersistableWidth r
                       => Config
                       -> Table r
-                      -> Restriction p r
+                      -> (Record Flat r -> Restrict (PlaceHolders p))
                       -> Update (r, p)
 typedUpdateAllColumn' config tbl r = typedUpdate' config tbl $ liftTargetAllColumn' r
 
--- | Make typed 'Update' from 'Table' and 'Restriction'.
+-- | Make typed 'Update' from 'Table' and 'Restrict' computation.
 --   Update target is all column.
 typedUpdateAllColumn :: PersistableWidth r
                      => Table r
-                     -> Restriction p r
+                     -> (Record Flat r -> Restrict (PlaceHolders p))
                      -> Update (r, p)
 typedUpdateAllColumn = typedUpdateAllColumn' defaultConfig
 
--- | Make typed 'Update' from 'Config', derived table and 'AssignStatement'.
+-- | Make typed 'Update' from 'Config', derived table and 'Restrict' computation.
 --   Update target is all column.
 updateAllColumn' :: (PersistableWidth r, TableDerivable r)
                  => Config
-                 -> RestrictedStatement r (PlaceHolders p)
+                 -> (Record Flat r -> Restrict (PlaceHolders p))
                  -> Update (r, p)
-updateAllColumn' config = typedUpdateAllColumn' config derivedTable .restriction'
+updateAllColumn' config = typedUpdateAllColumn' config derivedTable
 
 {-# DEPRECATED derivedUpdateAllColumn' "use `updateAllColumn'` instead of this." #-}
--- | Make typed 'Update' from 'Config', derived table and 'AssignStatement'.
---   Update target is all column.
+-- | Deprecated. use 'updateAllColumn''.
 derivedUpdateAllColumn' :: (PersistableWidth r, TableDerivable r)
                         => Config
-                        -> RestrictedStatement r (PlaceHolders p)
+                        -> (Record Flat r -> Restrict (PlaceHolders p))
                         -> Update (r, p)
 derivedUpdateAllColumn' = updateAllColumn'
 
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'.
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'Restrict' computation.
 --   Update target is all column.
 updateAllColumn :: (PersistableWidth r, TableDerivable r)
-                       => RestrictedStatement r (PlaceHolders p)
-                       -> Update (r, p)
+                => (Record Flat r -> Restrict (PlaceHolders p))
+                -> Update (r, p)
 updateAllColumn = updateAllColumn' defaultConfig
 
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'Restrict' computation
 --   without placeholder other than target table columns.
 --   Update target is all column.
 updateAllColumnNoPH :: (PersistableWidth r, TableDerivable r)
-                    => RestrictedStatement r ()
+                    => (Record Flat r -> Restrict ())
                     -> Update r
 updateAllColumnNoPH =
-  typedUpdate' defaultConfig derivedTable . liftTargetAllColumn . restriction
+  typedUpdate' defaultConfig derivedTable . (fmap peRight .) . liftTargetAllColumn' . ((>> return unitPH) .)
 
 {-# DEPRECATED derivedUpdateAllColumn "use `updateAllColumn` instead of this." #-}
--- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'.
---   Update target is all column.
+-- | Deprecated. use 'updateAllColumn'.
 derivedUpdateAllColumn :: (PersistableWidth r, TableDerivable r)
-                       => RestrictedStatement r (PlaceHolders p)
+                       => (Record Flat r -> Restrict (PlaceHolders p))
                        -> Update (r, p)
 derivedUpdateAllColumn = updateAllColumn
 
@@ -329,7 +334,7 @@ insertValue = insertValue' defaultConfig
 
 -- | Make typed 'Insert' from 'defaultConfig', derived table and monadic builded 'Register' object with no(unit) placeholder.
 insertValueNoPH :: TableDerivable r => Register r () -> Insert ()
-insertValueNoPH = insertValue . (return unitPH <*)
+insertValueNoPH = insertValue . (>> return unitPH)
 
 {-# DEPRECATED derivedInsertValue "use `insertValue` instead of this." #-}
 -- | Make typed 'Insert' from 'defaultConfig', derived table and monadic builded 'Register' object.
@@ -378,8 +383,12 @@ typedInsertQuery :: Table r -> Pi r r' -> Relation p r' -> InsertQuery p
 typedInsertQuery =  typedInsertQuery' defaultConfig
 
 -- | Table type inferred 'InsertQuery'.
+insertQuery' :: TableDerivable r => Config -> Pi r r' -> Relation p r' -> InsertQuery p
+insertQuery' config = typedInsertQuery' config derivedTable
+
+-- | Table type inferred 'InsertQuery' with 'defaultConfig'.
 insertQuery :: TableDerivable r => Pi r r' -> Relation p r' -> InsertQuery p
-insertQuery =  typedInsertQuery' defaultConfig derivedTable
+insertQuery = insertQuery' defaultConfig
 
 {-# DEPRECATED derivedInsertQuery "use `insertQuery` instead of this." #-}
 -- | Table type inferred 'InsertQuery'.
@@ -398,43 +407,42 @@ newtype Delete p = Delete { untypeDelete :: String }
 unsafeTypedDelete :: String -> Delete p
 unsafeTypedDelete =  Delete
 
--- | Make untyped delete SQL string from 'Table' and 'Restriction'.
-deleteSQL :: Config -> Table r -> Restriction p r -> String
-deleteSQL config tbl r = showStringSQL $ deletePrefixSQL tbl <> sqlWhereFromRestriction config tbl r
+-- | Make untyped delete SQL string from 'Table' and 'Restrict' computation.
+deleteSQL :: Config -> Table r -> (Record Flat r -> Restrict (PlaceHolders p)) -> String
+deleteSQL config tbl r = showStringSQL $ deleteFromRestriction config tbl r
 
--- | Make typed 'Delete' from 'Config', 'Table' and 'Restriction'.
-typedDelete' :: Config -> Table r -> Restriction p r -> Delete p
+-- | Make typed 'Delete' from 'Config', 'Table' and 'Restrict' computation.
+typedDelete' :: Config -> Table r -> (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
 typedDelete' config tbl r = unsafeTypedDelete $ deleteSQL config tbl r
 
 {-# DEPRECATED typedDelete "use `typedDelete' defaultConfig` instead of this." #-}
--- | Make typed 'Delete' from 'Table' and 'Restriction'.
-typedDelete :: Table r -> Restriction p r -> Delete p
+-- | Make typed 'Delete' from 'Table' and 'Restrict' computation.
+typedDelete :: Table r -> (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
 typedDelete =  typedDelete' defaultConfig
 
-restrictedTable :: TableDerivable r => Restriction p r -> Table r
+restrictedTable :: TableDerivable r => (Record Flat r -> Restrict (PlaceHolders p)) -> Table r
 restrictedTable =  const derivedTable
 
--- | Make typed 'Delete' from 'Config', derived table and 'RestrictContext'
-delete' :: TableDerivable r => Config -> RestrictedStatement r (PlaceHolders p) -> Delete p
-delete' config rc = typedDelete' config (restrictedTable rs) rs  where
-  rs = restriction' rc
+-- | Make typed 'Delete' from 'Config', derived table and 'Restrict' computation.
+delete' :: TableDerivable r => Config -> (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
+delete' config rc = typedDelete' config (restrictedTable rc) rc
 
 {-# DEPRECATED derivedDelete' "use `delete'` instead of this." #-}
--- | Make typed 'Delete' from 'Config', derived table and 'RestrictContext'
-derivedDelete' :: TableDerivable r => Config -> RestrictedStatement r (PlaceHolders p) -> Delete p
+-- | Make typed 'Delete' from 'Config', derived table and 'Restrict' computation.
+derivedDelete' :: TableDerivable r => Config -> (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
 derivedDelete' = delete'
 
--- | Make typed 'Delete' from 'defaultConfig', derived table and 'RestrictContext'
-delete :: TableDerivable r => RestrictedStatement r (PlaceHolders p) -> Delete p
+-- | Make typed 'Delete' from 'defaultConfig', derived table and 'Restrict' computation.
+delete :: TableDerivable r => (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
 delete = delete' defaultConfig
 
--- | Make typed 'Delete' from 'defaultConfig', derived table and 'RestrictContext' with no(unit) placeholder.
-deleteNoPH :: TableDerivable r => RestrictedStatement r () -> Delete ()
-deleteNoPH rf = delete $ (return unitPH <*) . rf
+-- | Make typed 'Delete' from 'defaultConfig', derived table and 'Restrict' computation with no(unit) placeholder.
+deleteNoPH :: TableDerivable r => (Record Flat r -> Restrict ()) -> Delete ()
+deleteNoPH rf = delete $ (>> return unitPH) . rf
 
 {-# DEPRECATED derivedDelete "use `delete` instead of this." #-}
--- | Make typed 'Delete' from 'defaultConfig', derived table and 'RestrictContext'
-derivedDelete :: TableDerivable r => RestrictedStatement r (PlaceHolders p) -> Delete p
+-- | Make typed 'Delete' from 'defaultConfig', derived table and 'Restrict' computation.
+derivedDelete :: TableDerivable r => (Record Flat r -> Restrict (PlaceHolders p)) -> Delete p
 derivedDelete = delete
 
 -- | Show delete SQL string
