@@ -3,14 +3,15 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE DataKinds #-}
 
 module Model where
 
 import GHC.Generics (Generic)
 import Data.Int (Int32, Int64)
 
-import Database.Relational.Query (defaultConfig)
-import Database.Relational.Query.TH (defineTable, makeRelationalRecordDefault, defineScalarDegree)
+import Database.Relational (defaultConfig)
+import Database.Relational.TH (defineTable, makeRelationalRecordDefault, defineScalarDegree)
 
 
 $(defineTable defaultConfig "TEST" "set_a"
